##-*- Mode: CPerl -*-

##======================================================================
package DDC::XS::CQueryOptions;
use strict;

package DDC::XS::CQueryOptions;
*setContextCount = __PACKAGE__->can('setContextSentencesCount');


1; ##-- be happy
